"use strict";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.setArguments = setArguments;
exports.expect = expect;
exports.read = read;
exports.end = end;
exports.strictOptions = strictOptions;
exports.strictOptionsEqual = strictOptionsEqual;
exports.options = options;
exports.optionsEqual = optionsEqual;
var argv = exports.argv = process.argv.slice(2, process.argv.length);

function findName(name, names) {

	var findedName = false,
	    array = false;

	names.some(function (fullName) {

		var keys, shirtName;

		if (typeof fullName == "string") {

			if (fullName == name) {
				findedName = fullName;
				return true;
			}

			return false;
		}

		var proto = Object.getPrototypeOf(fullName);

		if (proto == Object.prototype) {

			keys = fullName;
			fullName = Object.keys(keys)[0];
			shirtName = keys[fullName];

			if (fullName == name || shirtName == name) {
				findedName = fullName;
				return true;
			}

			return false;
		}

		if (proto == Array.prototype) {
			var _fullName = fullName;

			var _fullName2 = _slicedToArray(_fullName, 2);

			fullName = _fullName2[0];
			shirtName = _fullName2[1];

			if (fullName == name || shirtName == name) {
				array = true;
				findedName = fullName;
				return true;
			}

			return false;
		}

		return false;
	});

	return findedName ? { name: findedName, isArray: array } : false;
}

function setArguments() {
	argv.splice(0, argv.length);
	argv.push.apply(argv, arguments);
}

/**
 * Strict expectation one of given commands.
 *
 *     command-line-app install 
 *
 *     expect(
 *         {"install": "i"}, - fullname and shirtname  
 *         ["update", "u"],  - also fullname and shirtname  
 *         "info"            - only one variant of name
 *     )
 * 
 * @param  {...Object} names 
 * @return {String}    fullname
 */
function expect() {

	if (!argv.length) {
		throw new Error("Unexpected end of arguments.");
	}

	for (var _len = arguments.length, names = Array(_len), _key = 0; _key < _len; _key++) {
		names[_key] = arguments[_key];
	}

	var sourceKey = argv.shift(),
	    argument = findName(sourceKey, names);

	if (!argument) {
		throw new Error("Unexpected argument \"" + sourceKey + "\".");
	}

	return argument.name;
}

/**
 * Strict reading of argument.
 *
 *     command-line-app some-value 
 * 
 * @return {String} argument
 */
function read() {

	if (!argv.length) {
		throw new Error("Unexpected end of arguments.");
	}

	return argv.shift();
}

/**
 * Strict expectation of end.
 *
 */
function end() {

	if (argv.length) {
		throw new Error("Unexpected argument \"" + argv[0] + "\".");
	}
}

/**
 * Strict reading of flags and options.
 *
 *     command-line-app --output test -p es2015,react --verbose
 *
 *     strictOptions([
 *         ["another"]      - for flags array is same as object notation
 *         "verbose"        - only one variant of name
 *     ], [
 *         {"output": "o"}, - fullname and shirtname  
 *         ["plugins", "p"] - fullname and shirtname for array
 *     ])
 * 
 * @param  {...Object} flagsNames
 * @param  {...Object} optionsNames
 * @return {Object}    fullname-value pairs 
 */
function strictOptions(flagsNames, optionsNames) {

	if (!argv.length) {
		return {};
	}

	var options = {},
	    argument = argv[0];

	while (argv.length && (argument.indexOf("--") == 0 || argument.indexOf("-") == 0 && argument.length == 2)) {

		var sourceKey = argv.shift().replace(/^(--|-)/, ""),
		    flagKey = findName(sourceKey, flagsNames),
		    optionKey = findName(sourceKey, optionsNames);

		if (!flagKey && !optionKey) {
			throw new Error("Unexpected key \"" + sourceKey + "\".");
		}

		if (!flagKey && optionKey && !argv.length) {
			throw new Error("Unexpected end of arguments.");
		}

		var value = argv[0];

		if (optionKey && value.indexOf("--") != 0 && value.indexOf("-") != 0 && value.length != 2) {

			argv.shift();

			if (optionKey.isArray) {
				value = value.split(",").map(function (element) {
					return element.replace(/^['"]|["']$/g, "");
				});
			} else {
				value = value.replace(/^['"]|["']$/g, "");
			}

			options[optionKey.name] = value;
		} else if (flagKey) {

			options[flagKey.name] = true;
		} else {
			throw new Error("Unexpected key \"" + value + "\".");
		}

		argument = argv[0];
	}

	return options;
}

/**
 * Strict reading of options with equal sign. 
 * If option is provided without value it will interpreted as `true`.
 *
 *     command-line-app --output=test -p=es2015,react --verbose
 *
 *     strictOptionsEqual(
 *         {"output": "o"},  - fullname and shirtname  
 *         ["plugins", "p"], - fullname and shirtname for array
 *         "verbose"         - only one variant of name
 *     )
 * 
 * @param  {...Object} names 
 * @return {Object}    fullname-value pairs 
 */
function strictOptionsEqual() {

	if (!argv.length) {
		return {};
	}

	var options = {},
	    argument = argv[0];

	for (var _len2 = arguments.length, names = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
		names[_key2] = arguments[_key2];
	}

	while (argv.length && (argument.indexOf("--") == 0 || argument.indexOf("-") == 0)) {
		var _argv$shift$replace$s = argv.shift().replace(/^(--|-)/, "").split("=");

		var _argv$shift$replace$s2 = _slicedToArray(_argv$shift$replace$s, 2);

		var sourceKey = _argv$shift$replace$s2[0];
		var value = _argv$shift$replace$s2[1];
		var key = findName(sourceKey, names);

		if (!key) {
			throw new Error("Unexpected key \"" + sourceKey + "\".");
		}

		if (typeof value == "undefined") {
			value = true;
		} else if (key.isArray) {
			value = value.split(",").map(function (element) {
				return element.replace(/^['"]|["']$/g, "");
			});
		} else {
			value = value.replace(/^['"]|["']$/g, "");
		}

		options[key.name] = value;

		argument = argv[0];
	}

	return options;
}

/**
 * Unlimited reading of flags and options.
 *
 *     command-line-app --output test install -p es2015,react babel --verbose
 *
 *     options([
 *         ["another"]      - for flags array is same as object notation
 *         "verbose"        - only one variant of name
 *     ], [
 *         {"output": "o"}, - fullname and shirtname  
 *         ["plugins", "p"] - fullname and shirtname for array
 *     ])
 * 
 * @param  {...Object} flagsNames
 * @param  {...Object} optionsNames
 * @return {Object}    fullname-value pairs 
 */
function options(flagsNames, optionsNames) {

	if (!argv.length) {
		return {};
	}

	var options = {},
	    argvc = argv.slice(),
	    argc = argvc.length,
	    remove = [];

	for (var i = 0, argument = argvc[i]; i < argc; argument = argvc[++i]) {

		if (argument.indexOf("--") != 0 && (argument.indexOf("-") != 0 || argument.length != 2)) {
			continue;
		}

		var sourceKey = argument.replace(/^(--|-)/, ""),
		    flagKey = findName(sourceKey, flagsNames),
		    optionKey = findName(sourceKey, optionsNames);

		if (!flagKey && !optionKey) {
			throw new Error("Unexpected key \"" + sourceKey + "\".");
		}

		if (!flagKey && optionKey && i == argc - 1) {
			throw new Error("Unexpected end of arguments.");
		}

		var value = argvc[i + 1];

		if (optionKey && value.indexOf("--") != 0 && value.indexOf("-") != 0 && value.length != 2) {

			remove.unshift(i++);

			if (optionKey.isArray) {
				value = value.split(",").map(function (element) {
					return element.replace(/^['"]|["']$/g, "");
				});
			} else {
				value = value.replace(/^['"]|["']$/g, "");
			}

			options[optionKey.name] = value;
		} else if (flagKey) {

			options[flagKey.name] = true;
		} else {
			throw new Error("Unexpected key \"" + value + "\".");
		}

		remove.unshift(i);
	}

	remove.forEach(function (index) {
		return argv.splice(index, 1);
	});

	return options;
}

/**
 * Unlimited reading of with equal sign. 
 * If option is provided without value it will interpreted as `true`.
 *
 *     command-line-app --output=test install -p=es2015,react babel --verbose
 *
 *     optionsEqual(
 *         {"output": "o"},  - fullname and shirtname  
 *         ["plugins", "p"], - fullname and shirtname for array
 *         "verbose"         - only one variant of name
 *     )
 * 
 * @param  {...Object} names 
 * @return {Object}    fullname-value pairs 
 */
function optionsEqual() {

	if (!argv.length) {
		return {};
	}

	var options = {},
	    argvc = argv.slice(),
	    argc = argvc.length,
	    remove = [];

	for (var _len3 = arguments.length, names = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
		names[_key3] = arguments[_key3];
	}

	for (var i = 0, argument = argvc[i]; i < argc; argument = argvc[++i]) {

		if (argument.indexOf("-") != 0) {
			continue;
		}

		var _argument$replace$spl = argument.replace(/^(--|-)/, "").split("=");

		var _argument$replace$spl2 = _slicedToArray(_argument$replace$spl, 2);

		var sourceKey = _argument$replace$spl2[0];
		var value = _argument$replace$spl2[1];
		var key = findName(sourceKey, names);

		remove.unshift(i);

		if (!key) {
			throw new Error("Unexpected key \"" + sourceKey + "\".");
		}

		if (typeof value == "undefined") {
			value = true;
		} else if (key.isArray) {
			value = value.split(",").map(function (element) {
				return element.replace(/^['"]|["']$/g, "");
			});
		} else {
			value = value.replace(/^['"]|["']$/g, "");
		}

		options[key.name] = value;
	}

	remove.forEach(function (index) {
		return argv.splice(index, 1);
	});

	return options;
}